import tensorflow as tf

relu = tf.nn.relu
relu6 = tf.nn.relu6
crelu = tf.nn.crelu
elu = tf.nn.elu
selu = tf.nn.selu
softplus = tf.nn.softplus
softsign = tf.nn.softsign
softmax = tf.nn.softmax
sigmoid = tf.nn.sigmoid
tanh = tf.nn.tanh
