import tensorflow as tf

gradientdescent = GradientDescent = tf.train.GradientDescentOptimizer
adadelta = Adadelta = tf.train.AdadeltaOptimizer
adagrad = Adagrad = tf.train.AdagradOptimizer
adagradda = AdagradDA = tf.train.AdagradDAOptimizer
momentum = Momentum = tf.train.MomentumOptimizer
adam = Adam = tf.train.AdamOptimizer
ftrl = Ftrl = tf.train.FtrlOptimizer
proximalgradientdescent = ProximalGradientDescent = tf.train.ProximalGradientDescentOptimizer
proximaladagrad = ProximalAdagrad = tf.train.ProximalAdagradOptimizer
rmsprop = RMSProp = tf.train.RMSPropOptimizer
